/* File: rsg.c
 * -----------
 */
#include "rsg.h"
void free_string(void *elemAddr)
{
	assert(elemAddr);
	free(*((char**) elemAddr));
}

/**
 * Function: free_prod
 * ----------------------
 * free element in the product vector
 */
void free_prod(void *elemAddr)
{
	cvector** product_word_vector;
	assert(elemAddr);
	product_word_vector=(cvector**)elemAddr;
	//printf("free prod_word_vector address: %d\n",*product_word_vector);
	VectorDispose(*product_word_vector);
	m--;
	//printf("free prod_word_vector %d\n",m);

	free(*product_word_vector);
}
void free_def(void *elemAddr)
{

	definition* dp;
	assert(elemAddr);
	dp=(definition*)elemAddr;
	assert(dp->nt);
	assert(dp->prod);
	//free non terminal
	free (dp->nt);
	nt_count--;
	//free product vector
	VectorDispose(dp->prod);
	free(dp->prod);
	prod_count--;
	/*cout=dp->prod->ItemsCount;
	for (i=0;i<cout;i++)
	{
		VectorDelete(dp->prod,0);
	}
	VectorDispose(dp->prod);
	*/

}
int compare_def (const void * a, const void * b)
{
	
	return strcmp(((definition*)a)->nt,((definition*)b)->nt);
}

int safeToLower(int c)
{

	if (isupper(c))
		return(tolower(c));

	return(c);
}



int hashfn_def(const void *elemAddr,int TBLSZ)
{
	int ii=0, a=127;
	const char* pp=((definition*)elemAddr)->nt;
	//while (*pp) (ii==ii<<1^*pp++;
        for (;*pp!=0;pp++)
           ii=(a*ii+safeToLower(*pp))%TBLSZ;
	if (ii<0) ii=-ii;
	ii%=TBLSZ;
	return ii;
}
/* 
 * PrintWithWrap
 * -------------
 * This is structured like an fprintf function, you pass the FILE
 * to write to, the format string, and then any number of matchings
 * arguments for the format string.  The only difference is that it
 * internally tracks how many characters are on the current line and
 * when getting to close to the end, finds a convenient space character
 * and turns it into a newline to get new line-wrapping output.
 */
static void PrintWithWrap(FILE *out, const char *format, ...) 
{
    #define WRAP_AT_COLUMN 65
    static int column = 0;  // keeps track of how many characters on this line
    char buffer[2048];
    int i, len;
    va_list ap;

    va_start(ap, format);
    vsprintf(buffer, format, ap);
    va_end(ap);
    len = (int)strlen(buffer);
    
    for (i = 0; i < len; i++) {
	  if ((column > WRAP_AT_COLUMN) && isspace(((int)buffer[i]))) {
         fprintf(out, "\n");		// change space to return 
         column = 0;			// re-start counter
      } else {	// otherwise just print it normally and
           		// keep our internal column number in sync
        fputc(buffer[i], out);
        column++;
        if (buffer[i] == '\n') column = 0;
      }
   }
}




void GenerateSentences(const char* filename, int repeat)
{
	int i;
	bool discardDelimiters=false;
	hashset def_table;
	ReadGrammar(&def_table,NewScannerFromFilename(filename,DELIMITERS, discardDelimiters));
	for (i=0;i<repeat;i++)
	{
		PrintWithWrap(stdout, "Version #%d: ----------------------------\n   ", i);
		ExpandGrammar(&def_table);
		PrintWithWrap(stdout,"\n\n");
	}
 	DefinitionDispose(&def_table);
}






/* read nt and products in the definitions, add each definition into definition table
   leave DefinitionDispose to delete the product string spaces.
   */

void ReadGrammar(hashset *def_table, Scanner s)
{
	cvector* prod_word_vector=NULL;
	char word[MAX_WORD_LEN];
	char *nt,*newword;
	assert(s);
	DefinitionTableNew(def_table);
	assert(def_table);
	//vector to store product word


	while (SkipUntil(s, S_DEF)!=EOF)//read each definiton
	{
		//read each nt, product in the definiton
/*		SkipUntil(s,S_NT);
		// get the non terminal
		if (ReadNextToken(s, word, MAX_WORD_LEN))
		{
			nt=(char*)malloc(strlen(word)+1);
			nt_count++;
			strcpy(nt,word);
		}
		*/
		// read next Non Terminal
		nt=ReadNextNonTerminal(s,word);
		while (ReadNextToken(s, word, MAX_WORD_LEN)) {
			if (strcmp(word,E_DEF)==0)
				break;//end of a definition
			if (strcmp(word,E_PROD)!=0)//not EndOfProduct
			{

				if (!prod_word_vector)
				{
					prod_word_vector=(cvector*)malloc(sizeof(cvector));
					m++;
					//printf("new prod_word_vector %d\n",m);

					VectorNew(prod_word_vector, sizeof(char*), free_string, PROD_SIZE);
					assert(prod_word_vector);
				}	
				newword=NewString(word);
				VectorAppend(prod_word_vector,&newword);//append word to the product word vector
				//printf("add %s at address %d\n",newword,newword);
				//append word to product
				//product=StringCat(product,word);
			}
			else
			{
				//end of a product, add it to product vector
				//after got the product, append it to product vector
				//leave nt and product to be deleted in DefinitonDispose, the Free Functions will be called to free them
				DefinitionAdd(def_table, nt, prod_word_vector);
				//printf("add %d: %s\n",m,product);
				//assert(def_table->ItemsCount>0);
				prod_word_vector=NULL;
			}
		}
		assert(nt);
	}
	if (prod_word_vector)
	{
		VectorDispose(prod_word_vector);
		free(prod_word_vector);
		m--;
	}
	

	FreeScanner(s);

}
/* make another copy of str, it's client's repsonsiblity to free the returned char arrary*/
char* NewString(const char* str)
{
	char* newstring;
	assert(str);
	newstring=(char*)malloc(strlen(str)+1);
	strcpy(newstring,str);
	assert(newstring);
	return newstring;
}
void ExpandGrammar(hashset *def_table)
{
	GetGrammar(def_table,S_SEN);

}
void GetGrammar(hashset *def_table,char* nt)
{
	int i;
	char** word;
	cvector* prod_word_vector=GetProductRandom(def_table,nt);
	for (i=0;i<prod_word_vector->ItemsCount;i++)
	{
		word=(char**)VectorNth(prod_word_vector, i);
		if(*word[0]==S_NT_CHAR)
			GetGrammar(def_table,*word);

		else
			//print out the words.
			if(!isspace(*word[0]))
			{
				if (!ispunct(*word[0]))
					PrintWithWrap(stdout," %s",*word);
				else
					PrintWithWrap(stdout,"%s",*word);
			}
	}



}

//get Non Terminal from Scanner, word as a buffer

char* ReadNextNonTerminal(Scanner s, char* word)
{
	char* nt;
			//read each nt, product in the definiton
		SkipUntil(s,S_NT);
		// get the non terminal
		if (ReadNextToken(s, word, MAX_WORD_LEN))
		{
			nt=(char*)malloc(strlen(word)+1);
			nt_count++;
			strcpy(nt,word);
		}
		return nt;
}



void DefinitionTableNew(hashset *def_table)
{	
	HashSetNew(def_table,sizeof(definition),DEF_TBSZ, hashfn_def,compare_def,free_def);

}
/* add a definition to the definition table def_table 
   if the nt exists in the definition table , add the prod to the prod_vector
   otherwise, creat a new entry in the def_table, which create a nt char arrary
   and create a cvector* hold prod_vector
   */
void DefinitionAdd(hashset* def_table, const char* nt, const cvector* prod_word_vector)
{

	definition *def,def_key;
	assert(def_table&&nt&&prod_word_vector);
	def_key.nt=(char*)nt;
	def=(definition*)HashSetLookup(def_table,&def_key);
	if (def)//definition exists
	{	
		//printf("Definition exists: %s, product vector address: %d\n",def->nt,def->prod);
 		assert(def->prod->ItemsCount>=0);
		ProductAdd(def->prod, prod_word_vector);
	}
	else
	{
		DefinitionNew(def_table,nt,prod_word_vector);
	}

}
/*   otherwise, creat a new entry in the def_table, which create a nt char arrary
   and create a cvector* hold prod_vector
*/


/*   otherwise, creat a new entry in the def_table, which create a nt char arrary
   and create a cvector* hold prod_vector
*/
void DefinitionNew(hashset* def_table, const char* nt, const cvector* prod_word_vector)
{
	definition def_key;
	cvector* prod_vector=(cvector*)malloc(sizeof(cvector));
	prod_count++;
	def_key.nt=(char*)nt;
	def_key.prod=prod_vector;
	//vector to hold products
	//each products will also hold a product word cvector
	VectorNew(def_key.prod, sizeof(cvector*), free_prod, PROD_SIZE);
	HashSetEnter(def_table,&def_key);
	DefinitionAdd(def_table,nt,prod_word_vector);
}
void DefinitionDispose(hashset* def_table)
{
	assert(def_table);
	HashSetDispose(def_table);
}

/* add a product to the prod_vector*/
void ProductAdd(cvector* prod_vector, const cvector* prod_word_vector)
{
	//printf("add prod_word_vector address: %d\n",prod_word_vector);
	VectorAppend(prod_vector,&prod_word_vector);//append product to the product vector
}

/* lookup NT int the definition table, and return 
the prod_position th product in the product vector*/
cvector* GetProductRandom(hashset* def_table, const char* nt)
{	
	cvector** s=NULL;
	int range;
	definition def_key,*def;
	assert(def_table);
	assert(nt);
    def_key.nt=(char*)nt;

	def=(definition*)HashSetLookup(def_table,&def_key);
	if(def)
	{
		range=def->prod->ItemsCount-1;
		if (range<0) range=0;
		s=(cvector**)VectorNth(def->prod,RandomInteger(0,range));
	}
	return *s;
}



/* The StringCat function concatenates str1, and str2, and return the concatenated string. 
*/
char* StringCat(char* str1, const char* str2)
{
	assert(str2);
	if (!str1)//if str1 is null, alloc space for it
	{
		str1=(char*)malloc(strlen(str2)+1);
		assert(str1);
		strcpy(str1,str2);
	}
	else

	{
		str1=(char*)realloc(str1,strlen(str1)+strlen(str2)+1);
		assert(str1);
		strcpy(str1+strlen(str1),str2);
	}
	return str1;
}






















int main(int argc, char *argv[])
{
	if (argc < 2)
	{
		printf("\nOOPS!  This program takes one argument which is the Grammar file\n");
		printf("format: rsg input_grammar.g\n");
		return 0;
	}
	//Random Sentence Generation, REPEAT times.
	Randomize();
	GenerateSentences(argv[1],REPEAT);
/*
	for (i=0;i<1;i++)
		//GenerateSentences("E:\\project\\research\\rsg\\grammars\\excuse.g",REPEAT);
	GenerateSentences("grammars/excuse.g",REPEAT);
	*/
	return 0;
}

