#ifndef _rsg_
#define _rsg_

#include <stdio.h>
#include <stdarg.h> //for va_arg
#include <string.h>
#include <ctype.h>
#include <assert.h>
#include "random.h"
#include "scanner.h"
#include "cvector.h"
#include "hashtable.h"
// Grammar file skips white space...
#define DELIMITERS " \t\n\r"

// Useful "tokens"
#define E_PROD ";" //start of product
#define S_DEF "{" //start of definition
#define E_DEF "}" //end of definition
#define S_NT "<" //start of non terminal
#define S_NT_CHAR '<'
#define S_SEN "<start>" //start of sentence

// Some default values - definition per grammar file, words per production, length of each word
#define DEF_TBSZ 21 //size of Definition hash table
#define PROD_SIZE 11 //initial number of words of production vector
#define MAX_WORD_LEN 128 //max length of a word/token
#define REPEAT 3 // number of sentences to be generated.

/*RSG element*/
typedef struct definition{
	char* nt;
	cvector* prod;
} definition;

	static int m=0;
	static int nt_count=0;
	static int prod_count=0;


/*** Functions provided ***/
static void PrintWithWrap(FILE *out, const char *format, ...) ;

/****Functions coded by Zheyuan Yu **************
Last update: Dec 28, 2004
************************************************/
void GenerateSentences(const char* filename, int repeat);

/*read grammar file into definition_table */
void ReadGrammar(hashset *def_table, Scanner s);
//expand grammar
void ExpandGrammar(hashset *def_table);
//random get a grammar for Non Terminal;
void GetGrammar(hashset *def_table, char* nt);
//get Non Terminal from Scanner, word as a buffer
char* ReadNextNonTerminal(Scanner s, char* word);

/* initialize the definition hash table as followings:
   create a new Non Terminal Hash table, set the compare and free function
   */
void DefinitionTableNew(hashset* def_table);
/* add a definition to the definition table def_table 
   if the nt exists in the definition table , add the prod to the prod_vector
   otherwise, creat a new entry in the def_table, which create a nt char arrary
   and create a cvector* hold prod_vector
   */
void DefinitionAdd(hashset* def_table, const char* nt, const cvector* prod_word_vector);
/*   otherwise, creat a new entry in the def_table, which create a nt char arrary
   and create a cvector* hold prod_vector
*/
void DefinitionNew(hashset* def_table, const char* nt, const cvector* prod_word_vector);
/*dispose Definition Table and product vectors */
void DefinitionDispose(hashset* def_table);
/* add a product to the prod_vector*/
void ProductAdd(cvector* prod_vector, const cvector* prod_word_add);
/* lookup NT in the definition table, and return 
the prod_position th product in the product vector*/
cvector* GetProductRandom(hashset* def_table, const char* nt);

/* The StringCat function concatenates str1, and str2, and return the concatenated string. 
*/
char* StringCat(char* str1, const char* str2);
/* make another copy of str, it's client's repsonsiblity to free the returned char arrary*/
char* NewString(const char* str);
/* convert a byte to lower-case */
int safeToLower(int c);



#endif

