#include "scanner.h"
#include "hashtable.h"
#include <assert.h>
#include <stdlib.h>
#include <string.h>


int compare (const void * a, const void * b)
{
	return strcmp((char*)a,(char*)b);
}

int hashfn_string(const void *elemAddr,int TBLSZ)
{
	int ii=0, a=127;
	const char* pp=(char*)elemAddr;
	//while (*pp) (ii==ii<<1^*pp++;
        for (;*pp!=0;pp++)
           ii=(a*ii+*pp)%TBLSZ;
	if (ii<0) ii=-ii;
	ii%=TBLSZ;
	return ii;
}
/* create hash set to hold the delimiters. Hash set is ideal for fast search
return pointer h to the hash set
*/
void NewDelimiterDictionary(hashset* h, const char *delimiters)
{
	int i;
	int len=(int)strlen(delimiters);
	int elemSize=2, numBuckets=91;
	char c[2];
	c[1]='\0';
	HashSetNew(h,elemSize,numBuckets, hashfn_string,compare,NULL);
	for (i=0;i<len;i++)
	{
		c[0]=delimiters[i];
		HashSetEnter(h,c);
	}

}


typedef struct ScannerImplementation
{
	hashset delimeters;
	bool discardDelimiters;
	FILE* fp;
	bool file_open;/*indicator whether the scanner open a file */
} ScannerImplementation;

Scanner NewScannerFromFilename(const char *filename, const char *delimiters,
			       bool discardDelimiters)
{
	Scanner s;
	FILE* fp;
	assert(delimiters);
	if (!filename)
		return NULL;
	fp=fopen(filename,"r");
	if (!fp)
		return NULL;
	s=NewScannerFromFile(fp,delimiters,discardDelimiters);
	s->file_open=true;
	return s;

}

Scanner NewScannerFromFile(FILE *fp, const char *delimiters, 
			   bool discardDelimiters)
{
	Scanner s;
	if (!fp)
		return NULL;
	assert (delimiters);
	s=malloc(sizeof(ScannerImplementation));
	s->fp=fp;
	NewDelimiterDictionary(&(s->delimeters),delimiters);
	s->discardDelimiters=discardDelimiters;
	s->file_open=false; //set it to default false, and let NewScannerFromFileName to set it to true
	return s;
}

void FreeScanner(Scanner s)
{
	if ((s->file_open)&&(s->fp))
		fclose(s->fp);
	//dispose delimiters dictionary
	HashSetDispose(&(s->delimeters));
	free(s);


}

bool ReadNextToken(Scanner s, char buffer[], int bufLen)
{

	int index=0;
	char c[2];
	c[1]='\0';

	assert(buffer);
	assert(bufLen>1);
	while((c[0]=fgetc(s->fp))!=EOF)
	{
		if (HashSetLookup(&(s->delimeters),c)) //it is a delimiter
		{
    	//	printf("\n%c (%d) is a delimiter\n ",c[0],c[0]);
			if (index>0) //already accumulated a token, return it;
			{
				ungetc(c[0],s->fp);//unget the delimiter to input stream
				buffer[index]='\0';
				return true;
			}
			/* if the delimiter is the first character in the token
			   1. if discardDelimiters false, return the delimiter as a token
			   2. else ignore the delimiter, and get the next token.
			*/
			if (!s->discardDelimiters) //not discard the delimiter 
			{
				strcpy(buffer,c);
				return true;
			}

			return ReadNextToken(s,buffer,bufLen);

		}
		//not a delimiter, accumulate the token, until the buffer is full
	//	printf("\n%c (%d) not a delimiter\n ",c[0],c[0]);

		if (index<bufLen)
		{
			buffer[index++]=c[0];
		}
		else
		{
			ungetc(c[0],s->fp);
			break;
		}
	}
	if (index>0)
	{
		printf("token %s\n",buffer);
		return true;
	}
	else
		return false;

}
int SkipOver(Scanner s, const char *skipSet)
{
	char c;
	assert(skipSet);
	while ((c=getc(s->fp))!=EOF)
	{
		if (!strchr(skipSet, c))//should not skip it
		{
			ungetc(c,s->fp);
			return c;
		}
	}
	return EOF;
}

int SkipUntil(Scanner s, const char *untilSet)
{
	char c;
	assert(untilSet);
	while ((c=getc(s->fp))!=EOF)
	{
		if (strchr(untilSet, c))//find the until char
		{
			ungetc(c,s->fp);
			return c;
		}
	}
	return EOF;

}
/*
int main()
{
	return 0;
}
*/
